import React, { useRef, useImperativeHandle, useState } from 'react';
import { StyleSheet } from 'react-native';
import { Transition, Transitioning } from 'react-native-reanimated';
import { ColumnLayout, RowLayout, RowWrapLayout, ColumnWrapLayout } from './Layouts';
export let GalleryLayout;

(function (GalleryLayout) {
  GalleryLayout["Column"] = "column";
  GalleryLayout["ColumnWrap"] = "columnWrap";
  GalleryLayout["Row"] = "row";
  GalleryLayout["RowWrap"] = "rowWrap";
})(GalleryLayout || (GalleryLayout = {}));

const getLayout = layout => {
  switch (layout) {
    case GalleryLayout.Column:
      return ColumnLayout;

    case GalleryLayout.ColumnWrap:
      return ColumnWrapLayout;

    case GalleryLayout.Row:
      return RowLayout;

    case GalleryLayout.RowWrap:
      return RowWrapLayout;

    default:
      return RowLayout;
  }
};

const transition = /*#__PURE__*/React.createElement(Transition.Change, {
  durationMs: 400,
  interpolation: "easeInOut"
});

/** Component to show items in a Gallery style supporting multiple layouts and animating between them */
const GalleryView = React.forwardRef(({
  layout = GalleryLayout.RowWrap,
  spacing = 5,
  style,
  columnLayoutStyle,
  rowLayoutStyle,
  wrapLayoutStyle,
  itemStyle,
  children
}, ref) => {
  const [currentLayout, setCurrentLayout] = useState(layout);
  const transitionRef = useRef(null);
  useImperativeHandle(ref, () => ({
    setLayout: newLayout => {
      if (transitionRef.current) {
        transitionRef.current.animateNextTransition();
        setCurrentLayout(newLayout);
      }
    }
  }));
  let providedLayoutStyle = {};
  if (currentLayout === GalleryLayout.Column) providedLayoutStyle = columnLayoutStyle;
  if (currentLayout === GalleryLayout.Row) providedLayoutStyle = rowLayoutStyle;
  if (currentLayout === GalleryLayout.RowWrap) providedLayoutStyle = wrapLayoutStyle;
  return /*#__PURE__*/React.createElement(Transitioning.View, {
    style: [styles.container, getLayout(currentLayout).layout.container, style, providedLayoutStyle],
    ref: transitionRef,
    transition: transition
  }, React.Children.map(children, (currChild, index) => {
    if (currChild) {
      const childStyle = [currChild.props.style, getLayout(currentLayout).layout.child, {
        margin: spacing
      }, itemStyle];
      return React.cloneElement(currChild, {
        style: childStyle,
        key: "galleryItem".concat(index)
      });
    } else {
      return null;
    }
  }));
});
const styles = StyleSheet.create({
  container: {
    flex: 1
  }
});
export default GalleryView;
//# sourceMappingURL=GalleryView.js.map